package com.rjconsultores.ventaboletos.dao;

import java.util.Date;
import java.util.List;

import com.rjconsultores.ventaboletos.entidad.AbastoCentral;
import com.rjconsultores.ventaboletos.entidad.Aidf;
import com.rjconsultores.ventaboletos.entidad.Estacion;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;

public interface AbastoCentralDAO extends GenericDAO<AbastoCentral, Long> {

	
	public List<AbastoCentral> obtenerBilhetes(Aidf aidf, PuntoVenta origem);
	public List<AbastoCentral> obtenerBilhetes(Aidf aidf, String formInicial, String formFinal, PuntoVenta origem);
	
	public List<AbastoCentral> obtenerBilhetesPorPuntoVenta(PuntoVenta puntoVentaBilhetes);
	public List<AbastoCentral> obtenerBilhetesPorPuntoVenta(PuntoVenta puntoVentaBilhetes, Estacion origem);
	public List<AbastoCentral> obtenerBilhetesPorPuntoVenta(String formInicial, String formFinal, PuntoVenta puntoVentaBilhetes);
	public List<AbastoCentral> obtenerBilhetesPorPuntoVenta(String numSerie, String formInicial, String formFinal, PuntoVenta puntoVentaBilhetes, Estacion origem);
	
	public void actualizaBilhetes(List<AbastoCentral> bilhetes, PuntoVenta destino);
	public void actualizaBilhetes(List<AbastoCentral> bilhetes, Estacion destino);
	public void suscribirBilhetes(List<AbastoCentral> bilhetes);
	public List<AbastoCentral> buscaBilhetesPorAidf(Aidf aidf);
	
	public void gravaControleEstoqueMigracao(PuntoVenta puntoVenta, Date fecmodif, Integer usuarioId);
	
	public List<String> buscarSeriesBilhetesPorEstacion(Estacion estacion);
	
	public void borrarLosQueTienenAIDF(Aidf aidf);
	
	
}
